@@ -5,10 +5,17 @@ yt-dlp --help | make_readme.py
 This must be run in a console of correct width
 """
 
+# Allow direct execution
+import os
+import sys
+
+sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
 
 import functools
 import re
-import sys
+
+from devscripts.utils import read_file, write_file
 
 README_FILE = 'README.md'
 
@@ -38,6 +45,10 @@ switch_col_width = len(re.search(r'(?m)^\s{5,}', options).group())
 delim = f'\n{" " * switch_col_width}'
 
 PATCHES = (
+    (   # Standardize update message
+        r'(?m)^(    -U, --update\s+).+(\n    \s.+)*$',
+        r'\1Update this program to the latest version',
+    ),
     (  # Headings
         r'(?m)^  (\w.+\n)(    (?=\w))?',
         r'## \1'
@@ -63,12 +74,10 @@ PATCHES = (
     ),
 )
 
-with open(README_FILE, encoding='utf-8') as f:
-    readme = f.read()
+readme = read_file(README_FILE)
 
-with open(README_FILE, 'w', encoding='utf-8') as f:
-    f.write(''.join((
-        take_section(readme, end=f'## {OPTIONS_START}'),
-        functools.reduce(apply_patch, PATCHES, options),
-        take_section(readme, f'# {OPTIONS_END}'),
-    )))
+write_file(README_FILE, ''.join((
+    take_section(readme, end=f'## {OPTIONS_START}'),
+    functools.reduce(apply_patch, PATCHES, options),
+    take_section(readme, f'# {OPTIONS_END}'),
+)))
