@@ -51,12 +51,9 @@ class CallinIE(InfoExtractor):
         episode = next_data['props']['pageProps']['episode']
 
         id = episode['id']
-        title = (episode.get('title')
-                 or self._og_search_title(webpage, fatal=False)
-                 or self._html_extract_title(webpage))
+        title = episode.get('title') or self._generic_title('', webpage)
         url = episode['m3u8']
         formats = self._extract_m3u8_formats(url, display_id, ext='ts')
-        self._sort_formats(formats)
 
         show = traverse_obj(episode, ('show', 'title'))
         show_id = traverse_obj(episode, ('show', 'id'))
